#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
# Copyright (C) 2008 Siegfried-A. Gevatter
# Copyright (C) 2008 Markus Korn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import os.path
import logging
import ConfigParser
import xdg.BaseDirectory as basedir

from globals import Global

class Config:
    
    def __init__(self):
        
        self.config_file = basedir.load_first_config("leonov", "default.conf")
        if not self.config_file:
            self.config_file = os.path.join(basedir.save_config_path("leonov"),
                "default.conf")
        
        self.config = ConfigParser.SafeConfigParser()
        self.load()
        
    def _get(self, section, option, type=None, default=None):
        if self.config.has_section(section) and self.config.has_option(section, option):
            if type == bool:
                return self.config.getboolean(section, option)
            if type == int:
                return self.config.getint(section, option)
            if type == float:
                return self.config.getfloat(section, option)
            else:
                return self.config.get(section, option)
        else:
            logging.getLogger("configuration.Config._get").debug(
                "Debug: %s.%s does not exist.", section, option)
            if default is not None:
                return default
            else:
                return ""
    
    def _put(self, section, option, value):
        if not self.config.has_section(section):
            self.config.add_section(section)
        if hasattr(value, '__iter__'):
            value = ' '.join(value)
        self.config.set(section, option, str(value))
        self.changed_since_reload = True
    
    def load(self):
        self.config.read(self.config_file)
        self.changed_since_reload = False
    
    def save(self):
        if self.changed_since_reload:
            if not os.path.exists(self.config_file):
                fp=open(self.config_file,'w')
            else:
                fp=open(self.config_file,'r+')
                
            self.config.write(fp)
            fp.close()
            self.changed_since_reload = False
    
    def get_frontend(self):
        return self._get("default", "leonov_frontend").upper()
    
    def get_username(self):
        return self._get("launchpad", "lp_id")
    
    def get_cookie(self):
        return self._get("launchpad", "lp_cookie_file")

    def get_stay_authenticated(self):
        return self._get("launchpad", "lp_stay_authenticated", type=bool,
            default=False)

    def get_auth_per_day(self):
        return self._get("launchpad", "lp_auth_per_day", type=bool, default=False)
    
    def get_user_history(self):
        return [x for x in self._get("history", "users").split(" ") if x]

    def get_attachment_download_path( self ):
        return self._get( "launchpad", "lp_attachment_download_path" )
    
    def get_use_stable_launchpad(self):
        return self._get("launchpad","lp_use_stable", type=bool, default=False)
    
    def has_changed(self):
        return self.changed_since_reload
        
    def set_frontend(self, frontend_type):
        self._put("default", "leonov_frontend", frontend_type)
        
    def set_username(self, username):
        self._put("launchpad", "lp_id", username)
        
    def set_cookie(self, cookie_file):
        self._put("launchpad", "lp_cookie_file", cookie_file)
        
    def set_stay_authenticated(self, boolean):
        self._put("launchpad", "lp_stay_authenticated", boolean)

    def set_auth_per_day(self, boolean):
        self._put("launchpad", "lp_auth_per_day", boolean)
    
    def add_user_history(self, username):
        user_history = self.get_user_history()
        if username in user_history:
            # This is necessary to move the user to the end of the list
            user_history.remove(username)
        self._put("history", "users", user_history + [username])
    def set_attachment_download_path( self, download_path ):
        self._put( "launchpad", "lp_attachment_download_path", download_path )
        
    def set_use_stable_launchpad(self,stable=True):
        self._put("launchpad","lp_use_stable",stable)
        
config = Config()
