#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

try:
    import leonov.launchpadbugs.connector as Connector
    import leonov.launchpadbugs.exceptions as PyLpError
    from leonov.launchpadbugs.lpconstants import HTTPCONNECTION
    from launchpadbugs.basebuglistfilter import URLBugListFilter
except ImportError:
    import launchpadbugs.connector as Connector
    import launchpadbugs.exceptions as PyLpError
    from launchpadbugs.lpconstants import HTTPCONNECTION
    from launchpadbugs.basebuglistfilter import URLBugListFilter

    
import xdg.BaseDirectory as basedir
import os
import logging

class LPBugs:
    
    lpURL = "http://launchpad.net/"
    
    lpUserBugTypes = {
            0: '+assignedbugs',
            1: '+reportedbugs',
            2: '+commentedbugs',
            3: '+subscribedbugs',
        }
    
    def __init__(self, unstable=False):
        self.error = False
        self.bugListConnector = Connector.ConnectBugList()
        self.bugConnector = Connector.ConnectBug()
        self.projectConnector=Connector.ConnectProjectList()
        self.bugConnector.connection.set_username("Leonov/0.0.1")
        self.bugListConnector.connection.set_username("Leonov/0.0.1")
        self.projectConnector.connection.set_username("Leonov/0.0.1")
        self.unstable = unstable
        self.bugListConnector.connection.set_progress_hook(self.progress_hook)
        self.projectConnector.set_progress_hook(self.progress_hook)
        self.cookie_file = basedir.load_first_config("leonov", "lp_cookie.txt")
        if not self.cookie_file:
            self.cookie_file = os.path.join(basedir.save_config_path("leonov"),
                "lp_cookie.txt")
    
    def progress_hook(self, counter, block_size, size):
        """This method is virtual. You can overwrite it in your subclass."""
        pass
    
    def getBugConnector(self):
        return self.bugConnector
    
    def getBugListConnector(self):
        return self.bugListConnector
    
    def getProjectList(self):
        try:
            f=URLBugListFilter()
            f.add_option("batch",300)
            result=self.projectConnector(f(self.lpURL+"projects/+all"))
        except (PyLPError.LaunchpadZRLError, PyLpError.LaunchpadLoginError),e:
            self.throw_error(str(e))
            self.error=True
        else:
            return result

    def _bug_list(self, url):
        try:
            result = self.bugListConnector(url)
        except (PyLpError.LaunchpadURLError, PyLpError.LaunchpadLoginError), e:
            self.throw_error(str(e))
            self.error = True
        else:
            return result
    
    def getBugListForUser(self, lpUser, lpType=0):
        return self._bug_list(self.lpURL + "~" + lpUser + "/" + \
            self.lpUserBugTypes[lpType])

    def getMyBugList(self, lpType=0):
        return self._bug_list(self.lpURL + "people/+me/" + \
            self.lpUserBugTypes[lpType])
    
    def getErrorState(self):
        return self.error
    
    def reset_error_state(self):
        self.error = False
    
    def throw_error(self, e):
        """This method is virtual. You can overwrite it in your subclass.""" 
        pass

    def getBugReport(self, bugNo):
        self.bugReport = self.bugConnector(bugNo)
        return self.bugReport
    
    def getCookieFile(self):
        return self.cookie_file
        
    def setContentType(self, content_type):
        if content_type not in self.bugConnector.connection.content_types:
            self.bugConnector.connection.content_types.append(content_type)
    
    def authenticateLPUser(self, email, password):
        try:
            logging.getLogger("leonov.backend.lpbackend.authenticateLPUser"
                ).debug("Trying to log in with Launchpad...")
            self.bugListConnector.authentication = {
                    "email": email, 
                    "password": password,
                }
            self.bugListConnector.connection.save_cookie(self.cookie_file)
            os.chmod(self.cookie_file, 0600)
            self.bugConnector.authentication = self.cookie_file
            self.projectConnector.authentication=self.cookie_file
            if not self.unstable:
                logging.getLogger("leonov.backend.lpbackend").debug("LPBugs.__init__: unstable == %s ",self.unstable)
                self.bugListConnector.set_connection_mode(HTTPCONNECTION.MODE.STABLE)
                self.bugConnector.set_connection_mode(HTTPCONNECTION.MODE.STABLE)
                self.projectConnector.set_connection_mode(HTTPCONNECTION.MODE.STABLE)

        except PyLpError.LaunchpadLoginFailed, e:
            self.throw_error(str(e)) #thekorn: not sure about the type here
            self.error = True

    def authenticateLPUserViaCookie(self):
        try:
            logging.getLogger("leonov.backend.lpbackend.authenticateLPUsseViaCookie" 
                ).debug("Trying to log in to launchpad via cookie")
            self.bugListConnector.authentication=self.cookie_file
            self.bugConnector.authentication=self.cookie_file
            self.projectConnector.authentication=self.cookie_file
            if not self.unstable:
                logging.getLogger("leonov.backend.lpbackend").debug("LPBugs.__init__: unstable == %s ",self.unstable)
                self.bugListConnector.set_connection_mode(HTTPCONNECTION.MODE.STABLE)
                self.bugConnector.set_connection_mode(HTTPCONNECTION.MODE.STABLE)
                self.projectConnector.set_connection_mode(HTTPCONNECTION.MODE.STABLE)
            #thekorn: the following exception will never be raised
            #  there is no way to validate the given cookie for py-lp-bugs
        except PyLpError.LaunchpadLoginFailed, e:
            self.throw_error(e)
            self.error = True
