#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Nathan Samson
# Copyright (C) 2008 Siegfried-A. Gevatter
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import os
import gtk
import gobject

from leonov.backend import config, Global, common
from leonov.gtk.lpwidget import LPWidget
from leonov.gtk.bugsnotebook import BugsNotebook
from leonov.gtk.threads import run_thread

gtk.gdk.threads_init()

class frontend:
    
    def __init__(self):
        
        ui_file = common.first_existing_file("leonov/gtk/leonov-builder.ui",
            "/usr/share/leonov/leonov-builder.ui")
        
        gtk.about_dialog_set_url_hook(self._openBrowser)
        self._builder = gtk.Builder()
        self._builder.add_from_file(ui_file)
        self._builder.connect_signals(self)
        self._mainWindow = self._builder.get_object("mainWindow")
        self._mainWindow.show()
        
        self._builder.get_object("logoutMenuItem").set_sensitive(False)

        # Replace the GPL header with the full license text
        license_file = '/usr/share/common-licenses/GPL'
        if os.path.isfile(license_file):
            self._builder.get_object("aboutDialog").set_property("license",
                open(license_file).read())
        
        # Write the author names into the about dialog
        authors_file = '/usr/share/doc/leonov/AUTHORS'
        if os.path.isfile(authors_file):
            self._builder.get_object("aboutDialog").set_authors(
                [name.strip('\n') for name in open(authors_file).readlines()])
        
        # Fill the "Launchpad user" field with the default value
        user_history = config.get_user_history()
        if len(user_history):
            self._builder.get_object("userNameEntry").set_text(user_history[-1])
        
        # Setup auto-completion for the "Launchpad user" field
        completion = gtk.EntryCompletion()
        completion.set_model()
        completion.set_text_column(0)
        list_store = gtk.ListStore(gobject.TYPE_STRING)
        for username in config.get_user_history():
            list_store.append((username,))
        completion.set_model(list_store)
        self._builder.get_object("userNameEntry").set_completion(completion)
        del completion, list_store
        
        bugsWidget = BugsNotebook(self._builder)
        bugsWidget.show()
        self._builder.get_object("mainHPaned").add2(bugsWidget)

        self._LPWidget = LPWidget(bugsWidget, self._builder)

        # Show the account dialog
        #self._onLogin()
        if Global.cookie_file and os.path.isfile(Global.cookie_file):
            run_thread(self._login, [None, None, True])
    
        gtk.link_button_set_uri_hook(self._openBrowser)
        gtk.main()

    def _onAccountDialogResponse(self, obj, response):
        if response == 1:
            emailEntry = self._builder.get_object("accountEmailEntry")
            pwEntry = self._builder.get_object("accountPasswordEntry")
            run_thread(self._login, [emailEntry.get_text(), pwEntry.get_text()])
        elif response == 2:
            return #pass
        else:
            self._builder.get_object("accountDialog").hide()

    def _onLogin(self, object=None):
        self._builder.get_object("accountEmailEntry").set_text(
            config.get_username())
        self._builder.get_object("stayAuthenticated").set_active(
            int(config.get_stay_authenticated()))
        self._builder.get_object("accountDialog").run()

    def _onLogout(self, object=None):
        self._LPWidget.logout()
        self._builder.get_object("loginButton").set_sensitive(True)
        self._builder.get_object("loginMenuItem").set_sensitive(True)
        self._builder.get_object("logoutMenuItem").set_sensitive(False)

    def _login(self, email=None, password=None, cookie=False):
        gtk.gdk.threads_enter()
        self._builder.get_object("accountMessageLabel").show()
        self._builder.get_object("accountMessageLabel").set_text("Logging in...")
        self._builder.get_object("statusBar").push(0, "Logging in...")
        self._builder.get_object("accountDialogActionArea").set_sensitive(False)
        gtk.gdk.threads_leave()
        self._LPWidget.login(email, password, cookie)
        gtk.gdk.threads_enter()
        self._builder.get_object("statusBar").pop(0)
        self._builder.get_object("accountDialogActionArea").set_sensitive(True)
        
        if Global.thread.error:
            self._builder.get_object("accountMessageLabel").set_markup(
                "<b>"+str(Global.thread.error_message)+"</b>")
            self._builder.get_object("accountMessageLabel").show()
            self._builder.get_object("accountDialog").run()
        else:
            self._builder.get_object("loginButton").set_sensitive(False)
            self._builder.get_object("loginMenuItem").set_sensitive(False)
            self._builder.get_object("logoutMenuItem").set_sensitive(True)
            self._builder.get_object("accountMessageLabel").hide()
            self._builder.get_object("accountDialog").hide()
        gtk.gdk.threads_leave()
        
        if not Global.thread.error and not cookie:
            config.set_stay_authenticated(
                self._builder.get_object("stayAuthenticated").get_active())
            config.set_username(
                self._builder.get_object("accountEmailEntry").get_text())
    
    def _onShowTasklist(self, obj):
        self._builder.get_object("userNameEntry").select_region(0, -1)
        self._builder.get_object("userNameEntry").grab_focus()
        self._builder.get_object("showBugsErrorMessage").hide()
        self._builder.get_object("tasklistUserDialog").run()

    def _onTasklistUserDialogResponse(self, obj, response):
        self._builder.get_object("tasklistUserDialog").hide()
        if response == 1:
            self._LPWidget.loadBugsForUser(
                self._builder.get_object("userNameEntry").get_text())

    def _close(self, *discard):
        config.save()
        gtk.main_quit()
        common.kill_stderr()

    def _onFullscreenToggled(self, obj):
        if obj.get_active():
            self._mainWindow.fullscreen()
        else:
            self._mainWindow.unfullscreen()

    def _onAboutButtonClicked(self, obj):
        self._builder.get_object("aboutDialog").run()

    def _onAboutDialogResponse(self, response, data):
        self._builder.get_object("aboutDialog").hide()
    
    def _openBrowser(self, button, data = None):
        os.spawnlp (os.P_NOWAIT, "gnome-open", "gnome-open", button.get_uri())
    
    def _onShowBug(self, object):
        self._builder.get_object("showBugDialog").run()
    
    def _onShowBugDialogResponse(self, object, response):
        # TODO: Verift if the introduced value is a valid bug number.
        object.hide()
        if response == 1:
            str = self._builder.get_object("taskNoEntry").get_text().strip("#")
            self._LPWidget.openBug(int(str))
