#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Nathan Samson
# Copyright (C) 2008 Siegfried-A. Gevatter
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import os
import gtk
import threading

from leonov.backend import LPBugs, Global, config
from leonov.gtk.threads import run_thread
from leonov.gtk.buglisttreeviewwidget import BugListTreeViewWidget

class LPWidget(LPBugs):

    def __init__(self, bugsNotebook, builder):
        LPBugs.__init__(self)
        self._loggedIn = False

        self._bugListUser = BugListTreeViewWidget()
        self._bugListUser.connect("bug-selection-changed", self._bugSelectionChanged)
        self._bugListUser.connect("refresh-button-clicked", self._refreshBugList)
        self._bugListUser.connect("close-button-clicked", self._closeBugList)
        self._bugListUser.setLabel("Personal tasks")
        self._bugListUser.show()
        self._bugListUser.set_sensitive(False)
        builder.get_object("viewUsersBugList").set_active(True)
        builder.get_object("viewUsersBugList") \
               .connect("toggled", self._onToggledViewBugList, \
                        self._bugListUser)
        self._bugListOther = BugListTreeViewWidget()
        self._bugListOther.connect("bug-selection-changed", self._bugSelectionChanged)
        self._bugListOther.connect("refresh-button-clicked", self._refreshBugList)
        self._bugListOther.connect("close-button-clicked", self._closeBugList)
        self._bugListOther.set_sensitive(False)
        builder.get_object("viewOthersBugList") \
               .connect("toggled", self._onToggledViewBugList, \
                        self._bugListOther)
        self._bugsNotebook = bugsNotebook
        self._builder = builder
        self._builder.get_object("sidebarBugListVBox").pack_start(self._bugListUser)
        self._builder.get_object("sidebarBugListVBox").pack_end(self._bugListOther)
        self._progressBar = gtk.ProgressBar()
        self._builder.get_object("statusBar").pack_end(self._progressBar, False)
        self._curSelectedUser = None
        bugsNotebook.connect("show-reporters-task-list",
                             self._onShowReportersTaskListButtonClicked)

    def loadBugsForUser(self, user = None):
        if user:
            self._curSelectedUser = user
        run_thread(self._downloadBugs, [user])

    def progress_hook(self, counter, block_size, size):
        value = (block_size * counter / float(size))
        Global.progress.update(id(threading.currentThread()), value)
        self._progressBar.set_fraction(Global.progress.get_fraction())

    def throw_error(self, message):
        Global.thread.error = True
        Global.thread.error_message = message
    
    def login(self, user=None, password=None, cookie=False):
        if cookie:
            assert not user and not password
            self.authenticateLPUserViaCookie()
        else:
            assert user and password
            self.authenticateLPUser(user, password)
        if not self.error:
            self._loggedIn = True
            self._bugListUser.set_sensitive(True)
        self.reset_error_state()
    
    def logout(self):
        self._loggedIn = False
        self._bugListUser.set_sensitive(False)
        if os.path.isfile(self.cookie_file):
            os.remove(self.cookie_file)
    
    def openBug(self, bug):
        run_thread(self._downloadBugInfo, [bug])
    
    def _bugSelectionChanged(self, object, bug):
        run_thread(self._downloadBugInfo, [bug])
    
    def _downloadBugInfo(self, bug):
        gtk.gdk.threads_enter()

        if self._bugsNotebook.openCached(bug):
            gtk.gdk.threads_leave()
            return
        
        self._builder.get_object("statusBar").push(2,
            "Fetching information for bug #%d..." % bug)
        gtk.gdk.threads_leave()
        report = self.getBugReport(bug)
        gtk.gdk.threads_enter()
        self._bugsNotebook.openBug(report)
        self._builder.get_object("statusBar").pop(2)
        gtk.gdk.threads_leave()
    
    def _onShowReportersTaskListButtonClicked(self, object, reporter):
        run_thread(self._downloadBugs, [reporter])
    
    def _downloadBugs(self, user = None):
        if user:
            self._bugListOther.setLabel("Tasks for " + user)
            self._builder.get_object("viewOthersBugList").set_active(True)
            self._bugListOther.show()
        
        gtk.gdk.threads_enter()
        self._builder.get_object("statusBar").push(1, "Downloading bug list...")
        self._progressBar.show()
        gtk.gdk.threads_leave()
        
        all =  []
        
        for type in [0, 1, 2, 3]:
            if Global.thread.error:
                break
            if user:
                bugs = self.getBugListForUser(user, type)
            else:
                bugs = self.getMyBugList(type)
            if bugs:
                all.append(bugs)
        
        gtk.gdk.threads_enter()
        self._builder.get_object("showBugsButton").set_sensitive(True)
        self._builder.get_object("statusBar").pop(1)
        self._progressBar.hide()
        Global.progress.clear()
        
        if Global.thread.error:
            self._builder.get_object("userNameEntry").select_region(0, -1)
            self._builder.get_object("showBugsErrorMessage").set_markup(
                Global.thread.error_message)
            self._builder.get_object("showBugsErrorMessage").show()
            self._builder.get_object("tasklistUserDialog").run()
        else:
            username_box = self._builder.get_object("userNameEntry")
            username = username_box.get_text()
            if username not in config.get_user_history():
                username_box.get_completion().get_model().append((username,))
            config.add_user_history(username)
        self.reset_error_state()
        gtk.gdk.threads_leave()
        
        if user:
            self._bugListOther.loadBugsForUser(all, user)
            self._bugListOther.set_sensitive(True)
        else:
            self._bugListUser.loadBugsForUser(all, user)
            self._bugListUser.set_sensitive(True)

    def _refreshBugList(self, object, unusedParam):
        if object == self._bugListUser:
            self.loadBugsForUser()
        else:
            self.loadBugsForUser(self._curSelectedUser)
    
    def _closeBugList(self, object, unusedParam):
        object.hide()
        if object == self._bugListUser:
            self._builder.get_object("viewUsersBugList").set_active(False)
        else:
            self._builder.get_object("viewOthersBugList").set_active(False)
    
    def _onToggledViewBugList(self, object, targetWidget):
        if object.get_active():
            targetWidget.show()
        else:
            targetWidget.hide()
